@@ -1,33 +1,43 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2020 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #ifndef _RUBBERBANDSTRETCHER_H_
 #define _RUBBERBANDSTRETCHER_H_
     
-#define RUBBERBAND_VERSION "1.5.0-gpl"
+#define RUBBERBAND_VERSION "1.9.0"
 #define RUBBERBAND_API_MAJOR_VERSION 2
-#define RUBBERBAND_API_MINOR_VERSION 3
+#define RUBBERBAND_API_MINOR_VERSION 6
 
 #include <vector>
 #include <map>
+#include <cstddef>
 
 /**
  * @mainpage RubberBand
  * 
  * The Rubber Band API is contained in the single class
  * RubberBand::RubberBandStretcher.
- *
+ * 
  * Threading notes for real-time applications:
  * 
  * Multiple instances of RubberBandStretcher may be created and used
@@ -109,7 +119,9 @@ public:
      *   percussive event).  This, the default setting, usually
      *   results in a clear-sounding output; but it is not always
      *   consistent, and may cause interruptions in stable sounds
-     *   present at the same time as transient events.
+     *   present at the same time as transient events.  The
+     *   OptionDetector flags (below) can be used to tune this to some
+     *   extent.
      *
      *   \li \c OptionTransientsMixed - Reset component phases at the
      *   peak of each transient, outside a frequency range typical of
@@ -131,7 +143,7 @@ public:
      *
      *   \li \c OptionDetectorCompound - Use a general-purpose
      *   transient detector which is likely to be good for most
-     *   situations.
+     *   situations.  This is the default.
      *
      *   \li \c OptionDetectorPercussive - Detect percussive
      *   transients.  Note that this was the default and only option
@@ -165,7 +177,8 @@ public:
      *   determine its own threading model.  Usually this means using
      *   one processing thread per audio channel in offline mode if
      *   the stretcher is able to determine that more than one CPU is
-     *   available, and one thread only in realtime mode.
+     *   available, and one thread only in realtime mode.  This is the
+     *   defafult.
      *
      *   \li \c OptionThreadingNever - Never use more than one thread.
      *  
@@ -191,20 +204,33 @@ public:
      *   likely to result in a smoother sound at the expense of
      *   clarity and timing.
      *
-     * 8. Flags prefixed \c OptionFormant control the handling of
+     * 8. Flags prefixed \c OptionSmoothing control the use of
+     * window-presum FFT and time-domain smoothing.  These options may
+     * not be changed after construction.
+     *
+     *   \li \c OptionSmoothingOff - Do not use time-domain smoothing.
+     *   This is the default.
+     *
+     *   \li \c OptionSmoothingOn - Use time-domain smoothing.  This
+     *   will result in a softer sound with some audible artifacts
+     *   around sharp transients, but it may be appropriate for longer
+     *   stretches of some instruments and can mix well with
+     *   OptionWindowShort.
+     *
+     * 9. Flags prefixed \c OptionFormant control the handling of
      * formant shape (spectral envelope) when pitch-shifting.  These
      * options may be changed at any time.
      *
      *   \li \c OptionFormantShifted - Apply no special formant
      *   processing.  The spectral envelope will be pitch shifted as
-     *   normal.
+     *   normal.  This is the default.
      *
      *   \li \c OptionFormantPreserved - Preserve the spectral
      *   envelope of the unshifted signal.  This permits shifting the
      *   note frequency without so substantially affecting the
      *   perceived pitch profile of the voice or instrument.
      *
-     * 9. Flags prefixed \c OptionPitch control the method used for
+     * 10. Flags prefixed \c OptionPitch control the method used for
      * pitch shifting.  These options may be changed at any time.
      * They are only effective in realtime mode; in offline mode, the
      * pitch-shift method is fixed.
@@ -212,7 +238,7 @@ public:
      *   \li \c OptionPitchHighSpeed - Use a method with a CPU cost
      *   that is relatively moderate and predictable.  This may
      *   sound less clear than OptionPitchHighQuality, especially
-     *   for large pitch shifts. 
+     *   for large pitch shifts.  This is the default.
 
      *   \li \c OptionPitchHighQuality - Use the highest quality
      *   method for pitch shifting.  This method has a CPU cost
@@ -224,6 +250,26 @@ public:
      *   options, this avoids discontinuities when moving across the
      *   1.0 pitch scale in real-time; it also consumes more CPU than
      *   the others in the case where the pitch scale is exactly 1.0.
+     *
+     * 11. Flags prefixed \c OptionChannels control the method used for
+     * processing two-channel audio.  These options may not be changed
+     * after construction.
+     *
+     *   \li \c OptionChannelsApart - Each channel is processed
+     *   individually, though timing is synchronised and phases are
+     *   synchronised at transients (depending on the OptionTransients
+     *   setting).  This gives the highest quality for the individual
+     *   channels but a relative lack of stereo focus and unrealistic
+     *   increase in "width".  This is the default.
+     *
+     *   \li \c OptionChannelsTogether - The first two channels (where
+     *   two or more are present) are considered to be a stereo pair
+     *   and are processed in mid-side format; mid and side are
+     *   processed individually, with timing synchronised and phases
+     *   synchronised at transients (depending on the OptionTransients
+     *   setting).  This usually leads to better focus in the centre
+     *   but a loss of stereo space and width.  Any channels beyond
+     *   the first two are processed individually.
      */
     
     enum Option {
@@ -253,12 +299,20 @@ public:
         OptionWindowShort          = 0x00100000,
         OptionWindowLong           = 0x00200000,
 
+        OptionSmoothingOff         = 0x00000000,
+        OptionSmoothingOn          = 0x00800000,
+
         OptionFormantShifted       = 0x00000000,
         OptionFormantPreserved     = 0x01000000,
 
         OptionPitchHighSpeed       = 0x00000000,
         OptionPitchHighQuality     = 0x02000000,
-        OptionPitchHighConsistency = 0x04000000
+        OptionPitchHighConsistency = 0x04000000,
+
+        OptionChannelsApart        = 0x00000000,
+        OptionChannelsTogether     = 0x10000000,
+
+        // n.b. Options is int, so we must stop before 0x80000000
     };
 
     typedef int Options;
@@ -408,35 +462,35 @@ public:
     void setPitchOption(Options options);
 
     /**
-     * Tell the stretcher exactly how many input samples it will
+     * Tell the stretcher exactly how many input sample frames it will
      * receive.  This is only useful in Offline mode, when it allows
      * the stretcher to ensure that the number of output samples is
      * exactly correct.  In RealTime mode no such guarantee is
      * possible and this value is ignored.
+     *
+     * Note that the value of "samples" refers to the number of audio
+     * sample frames, which may be multi-channel, not the number of
+     * individual samples. (For example, one second of stereo audio
+     * sampled at 44100Hz yields a value of 44100 sample frames, not
+     * 88200.)  This rule applies throughout the Rubber Band API.
      */
     void setExpectedInputDuration(size_t samples);
 
-    /**
-     * Ask the stretcher how many audio sample frames should be
-     * provided as input in order to ensure that some more output
-     * becomes available.  Normal usage consists of querying this
-     * function, providing that number of samples to process(),
-     * reading the output using available() and retrieve(), and then
-     * repeating.
-     *
-     * Note that this value is only relevant to process(), not to
-     * study() (to which you may pass any number of samples at a time,
-     * and from which there is no output).
-     */
-     size_t getSamplesRequired() const;
-
     /**
      * Tell the stretcher the maximum number of sample frames that you
-     * will ever be passing in to a single process() call. If you
-     * don't call this function, the stretcher will assume that you
-     * never pass in more samples than getSamplesRequired() suggested
-     * you should.  You should not pass in more samples than that
-     * unless you have called setMaxProcessSize first.
+     * will ever be passing in to a single process() call.  If you
+     * don't call this, the stretcher will assume that you are calling
+     * getSamplesRequired() at each cycle and are never passing more
+     * samples than are suggested by that function.
+     *
+     * If your application has some external constraint that means you
+     * prefer a fixed block size, then your normal mode of operation
+     * would be to provide that block size to this function; to loop
+     * calling process() with that size of block; after each call to
+     * process(), test whether output has been generated by calling
+     * available(); and, if so, call retrieve() to obtain it.  See
+     * getSamplesRequired() for a more suitable operating mode for
+     * applications without such external constraints.
      *
      * This function may not be called after the first call to study()
      * or process().
@@ -444,9 +498,41 @@ public:
      * Note that this value is only relevant to process(), not to
      * study() (to which you may pass any number of samples at a time,
      * and from which there is no output).
+     *
+     * Note that the value of "samples" refers to the number of audio
+     * sample frames, which may be multi-channel, not the number of
+     * individual samples. (For example, one second of stereo audio
+     * sampled at 44100Hz yields a value of 44100 sample frames, not
+     * 88200.)  This rule applies throughout the Rubber Band API.
      */
     void setMaxProcessSize(size_t samples);
 
+    /**
+     * Ask the stretcher how many audio sample frames should be
+     * provided as input in order to ensure that some more output
+     * becomes available.
+     * 
+     * If your application has no particular constraint on processing
+     * block size and you are able to provide any block size as input
+     * for each cycle, then your normal mode of operation would be to
+     * loop querying this function; providing that number of samples
+     * to process(); and reading the output using available() and
+     * retrieve().  See setMaxProcessSize() for a more suitable
+     * operating mode for applications that do have external block
+     * size constraints.
+     *
+     * Note that this value is only relevant to process(), not to
+     * study() (to which you may pass any number of samples at a time,
+     * and from which there is no output).
+     *
+     * Note that the return value refers to the number of audio sample
+     * frames, which may be multi-channel, not the number of
+     * individual samples. (For example, one second of stereo audio
+     * sampled at 44100Hz yields a value of 44100 sample frames, not
+     * 88200.)  This rule applies throughout the Rubber Band API.
+     */
+     size_t getSamplesRequired() const;
+
     /**
      * Provide a set of mappings from "before" to "after" sample
      * numbers so as to enforce a particular stretch profile.  The
@@ -483,9 +569,16 @@ public:
      * blocks in individual study() calls, or as a single large block.
      *
      * "input" should point to de-interleaved audio data with one
-     * float array per channel.  "samples" supplies the number of
-     * audio sample frames available in "input".  If "samples" is
-     * zero, "input" may be NULL.
+     * float array per channel. Sample values are conventionally
+     * expected to be in the range -1.0f to +1.0f.  "samples" supplies
+     * the number of audio sample frames available in "input". If
+     * "samples" is zero, "input" may be NULL.
+     *
+     * Note that the value of "samples" refers to the number of audio
+     * sample frames, which may be multi-channel, not the number of
+     * individual samples. (For example, one second of stereo audio
+     * sampled at 44100Hz yields a value of 44100 sample frames, not
+     * 88200.)  This rule applies throughout the Rubber Band API.
      * 
      * Set "final" to true if this is the last block of data that will
      * be provided to study() before the first process() call.
@@ -496,6 +589,17 @@ public:
      * Provide a block of "samples" sample frames for processing.
      * See also getSamplesRequired() and setMaxProcessSize().
      *
+     * "input" should point to de-interleaved audio data with one
+     * float array per channel. Sample values are conventionally
+     * expected to be in the range -1.0f to +1.0f.  "samples" supplies
+     * the number of audio sample frames available in "input".
+     *
+     * Note that the value of "samples" refers to the number of audio
+     * sample frames, which may be multi-channel, not the number of
+     * individual samples. (For example, one second of stereo audio
+     * sampled at 44100Hz yields a value of 44100 sample frames, not
+     * 88200.)  This rule applies throughout the Rubber Band API.
+     *
      * Set "final" to true if this is the last block of input data.
      */
     void process(const float *const *input, size_t samples, bool final);
@@ -510,6 +614,12 @@ public:
      * enough data has yet been processed.  Call getSamplesRequired()
      * to discover whether more input is needed.
      *
+     * Note that the return value refers to the number of audio sample
+     * frames, which may be multi-channel, not the number of
+     * individual samples. (For example, one second of stereo audio
+     * sampled at 44100Hz yields a value of 44100 sample frames, not
+     * 88200.)  This rule applies throughout the Rubber Band API.
+     *
      * This function returns -1 if all data has been fully processed
      * and all output read, and the stretch process is now finished.
      */
@@ -521,6 +631,13 @@ public:
      * channel for de-interleaved audio data) pointed to by "output".
      * The return value is the actual number of sample frames
      * retrieved.
+     *
+     * Note that the value of "samples" and the return value refer to
+     * the number of audio sample frames, which may be multi-channel,
+     * not the number of individual samples. (For example, one second
+     * of stereo audio sampled at 44100Hz yields a value of 44100
+     * sample frames, not 88200.)  This rule applies throughout the
+     * Rubber Band API.
      */
     size_t retrieve(float *const *output, size_t samples) const;
 
