@@ -1,22 +1,31 @@
 /* -*- c-basic-offset: 4 indent-tabs-mode: nil -*-  vi:set ts=8 sts=4 sw=4: */
 
 /*
-    Rubber Band
+    Rubber Band Library
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2010 Chris Cannam.
-    
+    Copyright 2007-2020 Particular Programs Ltd.
+
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
     published by the Free Software Foundation; either version 2 of the
     License, or (at your option) any later version.  See the file
     COPYING included with this distribution for more information.
+
+    Alternatively, if you have a valid commercial licence for the
+    Rubber Band Library obtained by agreement with the copyright
+    holders, you may redistribute and/or modify it under the terms
+    described in that licence.
+
+    If you wish to distribute code using the Rubber Band Library
+    under terms other than those of the GNU General Public License,
+    you must obtain a valid commercial licence before doing so.
 */
 
 #include "StretcherImpl.h"
 
-#include "dsp/PercussiveAudioCurve.h"
-#include "dsp/HighFrequencyAudioCurve.h"
-#include "dsp/ConstantAudioCurve.h"
+#include "audiocurves/PercussiveAudioCurve.h"
+#include "audiocurves/HighFrequencyAudioCurve.h"
+#include "audiocurves/ConstantAudioCurve.h"
 
 #include "StretchCalculator.h"
 #include "StretcherChannelData.h"
@@ -24,16 +33,14 @@
 #include "dsp/Resampler.h"
 #include "base/Profiler.h"
 #include "system/VectorOps.h"
-
-#ifndef _WIN32
-#include <alloca.h>
-#endif
+#include "system/sysutils.h"
 
 #include <cassert>
 #include <cmath>
 #include <set>
 #include <map>
 #include <deque>
+#include <algorithm>
 
 using namespace RubberBand;
 
@@ -42,6 +49,8 @@ using std::endl;
 
 namespace RubberBand {
 
+#ifndef NO_THREADING
+
 RubberBandStretcher::Impl::ProcessThread::ProcessThread(Impl *s, size_t c) :
     m_s(s),
     m_channel(c),
@@ -116,6 +125,8 @@ RubberBandStretcher::Impl::ProcessThread::abandon()
     m_abandoning = true;
 }
 
+#endif
+
 bool
 RubberBandStretcher::Impl::resampleBeforeStretching() const
 {
@@ -132,10 +143,33 @@ RubberBandStretcher::Impl::resampleBeforeStretching() const
         return (m_pitchScale > 1.0); // better performance
     }
 }
+
+void
+RubberBandStretcher::Impl::prepareChannelMS(size_t c,
+                                            const float *const *inputs,
+                                            size_t offset,
+                                            size_t samples, 
+                                            float *prepared)
+{
+    for (size_t i = 0; i < samples; ++i) {
+        float left = inputs[0][i + offset];
+        float right = inputs[1][i + offset];
+        float mid = (left + right) / 2;
+        float side = (left - right) / 2;
+        if (c == 0) {
+            prepared[i] = mid;
+        } else {
+            prepared[i] = side;
+        }
+    }
+}
     
 size_t
-RubberBandStretcher::Impl::consumeChannel(size_t c, const float *input,
-                                          size_t samples, bool final)
+RubberBandStretcher::Impl::consumeChannel(size_t c,
+                                          const float *const *inputs,
+                                          size_t offset,
+                                          size_t samples,
+                                          bool final)
 {
     Profiler profiler("RubberBandStretcher::Impl::consumeChannel");
 
@@ -147,6 +181,12 @@ RubberBandStretcher::Impl::consumeChannel(size_t c, const float *input,
 
     bool resampling = resampleBeforeStretching();
 
+    const float *input = 0;
+
+    bool useMidSide = ((m_options & OptionChannelsTogether) &&
+                       (m_channels >= 2) &&
+                       (c < 2));
+
     if (resampling) {
 
         toWrite = int(ceil(samples / m_pitchScale));
@@ -162,6 +202,20 @@ RubberBandStretcher::Impl::consumeChannel(size_t c, const float *input,
             cd.setResampleBufSize(reqSize);
         }
 
+#ifndef NO_THREADING
+#if defined HAVE_IPP && !defined USE_SPEEX
+        if (m_threaded) {
+            m_resamplerMutex.lock();
+        }
+#endif
+#endif
+
+        if (useMidSide) {
+            prepareChannelMS(c, inputs, offset, samples, cd.ms);
+            input = cd.ms;
+        } else {
+            input = inputs[c] + offset;
+        }
 
         toWrite = cd.resampler->resample(&input,
                                          &cd.resamplebuf,
@@ -169,6 +223,13 @@ RubberBandStretcher::Impl::consumeChannel(size_t c, const float *input,
                                          1.0 / m_pitchScale,
                                          final);
 
+#ifndef NO_THREADING
+#if defined HAVE_IPP && !defined USE_SPEEX
+        if (m_threaded) {
+            m_resamplerMutex.unlock();
+        }
+#endif
+#endif
     }
 
     if (writable < toWrite) {
@@ -179,10 +240,20 @@ RubberBandStretcher::Impl::consumeChannel(size_t c, const float *input,
     }
 
     if (resampling) {
+
         inbuf.write(cd.resamplebuf, toWrite);
         cd.inCount += samples;
         return samples;
+
     } else {
+
+        if (useMidSide) {
+            prepareChannelMS(c, inputs, offset, toWrite, cd.ms);
+            input = cd.ms;
+        } else {
+            input = inputs[c] + offset;
+        }
+
         inbuf.write(input, toWrite);
         cd.inCount += toWrite;
         return toWrite;
@@ -205,18 +276,23 @@ RubberBandStretcher::Impl::processChunks(size_t c, bool &any, bool &last)
     last = false;
     any = false;
 
+    float *tmp = 0;
+
     while (!last) {
 
         if (!testInbufReadSpace(c)) {
-//            cerr << "not enough input" << endl;
+            if (m_debugLevel > 2) {
+                cerr << "processChunks: out of input" << endl;
+            }
             break;
         }
 
         any = true;
 
         if (!cd.draining) {
-            size_t got = cd.inbuf->peek(cd.fltbuf, m_windowSize);
-            assert(got == m_windowSize || cd.inputSize >= 0);
+            size_t ready = cd.inbuf->getReadSpace();
+            assert(ready >= m_aWindowSize || cd.inputSize >= 0);
+            cd.inbuf->peek(cd.fltbuf, std::min(ready, m_aWindowSize));
             cd.inbuf->skip(m_increment);
         }
 
@@ -224,20 +300,20 @@ RubberBandStretcher::Impl::processChunks(size_t c, bool &any, bool &last)
         size_t phaseIncrement, shiftIncrement;
         getIncrements(c, phaseIncrement, shiftIncrement, phaseReset);
 
-        if (shiftIncrement <= m_windowSize) {
+        if (shiftIncrement <= m_aWindowSize) {
             analyseChunk(c);
             last = processChunkForChannel
                 (c, phaseIncrement, shiftIncrement, phaseReset);
         } else {
-            size_t bit = m_windowSize/4;
+            size_t bit = m_aWindowSize/4;
             if (m_debugLevel > 1) {
                 cerr << "channel " << c << " breaking down overlong increment " << shiftIncrement << " into " << bit << "-size bits" << endl;
             }
+            if (!tmp) tmp = allocate<float>(m_aWindowSize);
             analyseChunk(c);
-            float *tmp = (float *)alloca(m_windowSize * sizeof(float));
-            v_copy(tmp, cd.fltbuf, m_windowSize);
+            v_copy(tmp, cd.fltbuf, m_aWindowSize);
             for (size_t i = 0; i < shiftIncrement; i += bit) {
-                v_copy(cd.fltbuf, tmp, m_windowSize);
+                v_copy(cd.fltbuf, tmp, m_aWindowSize);
                 size_t thisIncrement = bit;
                 if (i + thisIncrement > shiftIncrement) {
                     thisIncrement = shiftIncrement - i;
@@ -253,6 +329,8 @@ RubberBandStretcher::Impl::processChunks(size_t c, bool &any, bool &last)
             cerr << "channel " << c << ": last = " << last << ", chunkCount = " << cd.chunkCount << endl;
         }
     }
+
+    if (tmp) deallocate(tmp);
 }
 
 bool
@@ -267,11 +345,17 @@ RubberBandStretcher::Impl::processOneChunk()
     // This is the normal process method in RT mode.
 
     for (size_t c = 0; c < m_channels; ++c) {
-        if (!testInbufReadSpace(c)) return false;
+        if (!testInbufReadSpace(c)) {
+            if (m_debugLevel > 2) {
+                cerr << "processOneChunk: out of input" << endl;
+            }
+            return false;
+        }
         ChannelData &cd = *m_channelData[c];
         if (!cd.draining) {
-            size_t got = cd.inbuf->peek(cd.fltbuf, m_windowSize);
-            assert(got == m_windowSize || cd.inputSize >= 0);
+            size_t ready = cd.inbuf->getReadSpace();
+            assert(ready >= m_aWindowSize || cd.inputSize >= 0);
+            cd.inbuf->peek(cd.fltbuf, std::min(ready, m_aWindowSize));
             cd.inbuf->skip(m_increment);
             analyseChunk(c);
         }
@@ -302,7 +386,7 @@ RubberBandStretcher::Impl::testInbufReadSpace(size_t c)
 
     size_t rs = inbuf.getReadSpace();
 
-    if (rs < m_windowSize && !cd.draining) {
+    if (rs < m_aWindowSize && !cd.draining) {
             
         if (cd.inputSize == -1) {
 
@@ -313,11 +397,18 @@ RubberBandStretcher::Impl::testInbufReadSpace(size_t c)
             // its input -- and that would give incorrect output, as
             // we know there is more input to come.
 
+#ifndef NO_THREADING
             if (!m_threaded) {
-//                cerr << "WARNING: RubberBandStretcher: read space < chunk size ("
-//                          << inbuf.getReadSpace() << " < " << m_windowSize
-//                          << ") when not all input written, on processChunks for channel " << c << endl;
+#endif
+                if (m_debugLevel > 1) {
+                    cerr << "WARNING: RubberBandStretcher: read space < chunk size ("
+                         << inbuf.getReadSpace() << " < " << m_aWindowSize
+                         << ") when not all input written, on processChunks for channel " << c << endl;
+                }
+
+#ifndef NO_THREADING
             }
+#endif
             return false;
         }
         
@@ -328,7 +419,7 @@ RubberBandStretcher::Impl::testInbufReadSpace(size_t c)
             }
             return false;
 
-        } else if (rs < m_windowSize/2) {
+        } else if (rs < m_aWindowSize/2) {
 
             if (m_debugLevel > 1) {
                 cerr << "read space = " << rs << ", setting draining true" << endl;
@@ -356,7 +447,7 @@ RubberBandStretcher::Impl::processChunkForChannel(size_t c,
 
     if (phaseReset && (m_debugLevel > 1)) {
         cerr << "processChunkForChannel: phase reset found, incrs "
-                  << phaseIncrement << ":" << shiftIncrement << endl;
+             << phaseIncrement << ":" << shiftIncrement << endl;
     }
 
     ChannelData &cd = *m_channelData[c];
@@ -368,16 +459,16 @@ RubberBandStretcher::Impl::processChunkForChannel(size_t c,
         // to write from the existing accumulator into the output.
         
         // We know we have enough samples available in m_inbuf --
-        // this is usually m_windowSize, but we know that if fewer
+        // this is usually m_aWindowSize, but we know that if fewer
         // are available, it's OK to use zeroes for the rest
         // (which the ring buffer will provide) because we've
         // reached the true end of the data.
         
-        // We need to peek m_windowSize samples for processing, and
+        // We need to peek m_aWindowSize samples for processing, and
         // then skip m_increment to advance the read pointer.
 
         modifyChunk(c, phaseIncrement, phaseReset);
-        synthesiseChunk(c); // reads from cd.mag, cd.phase
+        synthesiseChunk(c, shiftIncrement); // reads from cd.mag, cd.phase
 
         if (m_debugLevel > 2) {
             if (phaseReset) {
@@ -474,7 +565,7 @@ RubberBandStretcher::Impl::calculateIncrements(size_t &phaseIncrementRtn,
         }
     }
 
-    const int hs = m_windowSize/2 + 1;
+    const int hs = m_fftSize/2 + 1;
 
     // Normally we would mix down the time-domain signal and apply a
     // single FFT, or else mix down the Cartesian form of the
@@ -491,27 +582,41 @@ RubberBandStretcher::Impl::calculateIncrements(size_t &phaseIncrementRtn,
 
     if (m_channels == 1) {
 
-        df = m_phaseResetAudioCurve->processDouble(cd.mag, m_increment);
-        silent = (m_silentAudioCurve->processDouble(cd.mag, m_increment) > 0.f);
+        if (sizeof(process_t) == sizeof(double)) {
+            df = m_phaseResetAudioCurve->processDouble((double *)cd.mag, m_increment);
+            silent = (m_silentAudioCurve->processDouble((double *)cd.mag, m_increment) > 0.f);
+        } else {
+            df = m_phaseResetAudioCurve->processFloat((float *)cd.mag, m_increment);
+            silent = (m_silentAudioCurve->processFloat((float *)cd.mag, m_increment) > 0.f);
+        }
 
     } else {
 
-        double *tmp = (double *)alloca(hs * sizeof(double));
+        process_t *tmp = (process_t *)alloca(hs * sizeof(process_t));
 
         v_zero(tmp, hs);
         for (size_t c = 0; c < m_channels; ++c) {
             v_add(tmp, m_channelData[c]->mag, hs);
         }
-    
-        df = m_phaseResetAudioCurve->processDouble(tmp, m_increment);
-        silent = (m_silentAudioCurve->processDouble(tmp, m_increment) > 0.f);
+
+        if (sizeof(process_t) == sizeof(double)) {
+            df = m_phaseResetAudioCurve->processDouble((double *)tmp, m_increment);
+            silent = (m_silentAudioCurve->processDouble((double *)tmp, m_increment) > 0.f);
+        } else {
+            df = m_phaseResetAudioCurve->processFloat((float *)tmp, m_increment);
+            silent = (m_silentAudioCurve->processFloat((float *)tmp, m_increment) > 0.f);
+        }
     }
 
     int incr = m_stretchCalculator->calculateSingle
-            (getEffectiveRatio(), df, m_increment);
+        (getEffectiveRatio(), df, m_increment);
 
-    m_lastProcessPhaseResetDf.write(&df, 1);
-    m_lastProcessOutputIncrements.write(&incr, 1);
+    if (m_lastProcessPhaseResetDf.getWriteSpace() > 0) {
+        m_lastProcessPhaseResetDf.write(&df, 1);
+    }
+    if (m_lastProcessOutputIncrements.getWriteSpace() > 0) {
+        m_lastProcessOutputIncrements.write(&incr, 1);
+    }
 
     if (incr < 0) {
         phaseReset = true;
@@ -544,7 +649,7 @@ RubberBandStretcher::Impl::calculateIncrements(size_t &phaseIncrementRtn,
     if (silent) ++m_silentHistory;
     else m_silentHistory = 0;
 
-    if (m_silentHistory >= int(m_windowSize / m_increment) && !phaseReset) {
+    if (m_silentHistory >= int(m_aWindowSize / m_increment) && !phaseReset) {
         phaseReset = true;
         if (m_debugLevel > 1) {
             cerr << "calculateIncrements: phase reset on silence (silent history == "
@@ -634,46 +739,19 @@ RubberBandStretcher::Impl::analyseChunk(size_t channel)
 {
     Profiler profiler("RubberBandStretcher::Impl::analyseChunk");
 
-    int i;
-
     ChannelData &cd = *m_channelData[channel];
 
-    double *const R__ dblbuf = cd.dblbuf;
+    process_t *const R__ dblbuf = cd.dblbuf;
     float *const R__ fltbuf = cd.fltbuf;
 
-    int sz = m_windowSize;
-    int hs = m_windowSize/2;
+    // cd.fltbuf is known to contain m_aWindowSize samples
 
-    // cd.fltbuf is known to contain m_windowSize samples
-
-    m_window->cut(fltbuf);
-
-    if (cd.oversample > 1) {
-
-        int bufsiz = sz * cd.oversample;
-        int offset = (bufsiz - sz) / 2;
-
-        // eek
-
-        for (i = 0; i < offset; ++i) {
-            dblbuf[i] = 0.0;
-        }
-        for (i = 0; i < offset; ++i) {
-            dblbuf[bufsiz - i - 1] = 0.0;
-        }
-        for (i = 0; i < sz; ++i) {
-            dblbuf[offset + i] = fltbuf[i];
-        }
-        for (i = 0; i < bufsiz / 2; ++i) {
-            double tmp = dblbuf[i];
-            dblbuf[i] = dblbuf[i + bufsiz/2];
-            dblbuf[i + bufsiz/2] = tmp;
-        }
-    } else {
-        v_convert(dblbuf, fltbuf + hs, hs);
-        v_convert(dblbuf + hs, fltbuf, hs);
+    if (m_aWindowSize > m_fftSize) {
+        m_afilter->cut(fltbuf);
     }
 
+    cutShiftAndFold(dblbuf, m_fftSize, fltbuf, m_awindow);
+
     cd.fft->forwardPolar(dblbuf, cd.mag, cd.phase);
 }
 
@@ -690,16 +768,15 @@ RubberBandStretcher::Impl::modifyChunk(size_t channel,
         cerr << "phase reset: leaving phases unmodified" << endl;
     }
 
-    const double rate = m_sampleRate;
-    const int sz = m_windowSize;
-    const int count = (sz * cd.oversample) / 2;
+    const process_t rate = m_sampleRate;
+    const int count = m_fftSize / 2;
 
     bool unchanged = cd.unchanged && (outputIncrement == m_increment);
     bool fullReset = phaseReset;
     bool laminar = !(m_options & OptionPhaseIndependent);
     bool bandlimited = (m_options & OptionTransientsMixed);
-    int bandlow = lrint((150 * sz * cd.oversample) / rate);
-    int bandhigh = lrint((1000 * sz * cd.oversample) / rate);
+    int bandlow = lrint((150 * m_fftSize) / rate);
+    int bandhigh = lrint((1000 * m_fftSize) / rate);
 
     float freq0 = m_freq0;
     float freq1 = m_freq1;
@@ -717,27 +794,27 @@ RubberBandStretcher::Impl::modifyChunk(size_t channel,
         }
     }
 
-    int limit0 = lrint((freq0 * sz * cd.oversample) / rate);
-    int limit1 = lrint((freq1 * sz * cd.oversample) / rate);
-    int limit2 = lrint((freq2 * sz * cd.oversample) / rate);
+    int limit0 = lrint((freq0 * m_fftSize) / rate);
+    int limit1 = lrint((freq1 * m_fftSize) / rate);
+    int limit2 = lrint((freq2 * m_fftSize) / rate);
 
     if (limit1 < limit0) limit1 = limit0;
     if (limit2 < limit1) limit2 = limit1;
     
-    double prevInstability = 0.0;
+    process_t prevInstability = 0.0;
     bool prevDirection = false;
 
-    double distance = 0.0;
-    const double maxdist = 8.0;
+    process_t distance = 0.0;
+    const process_t maxdist = 8.0;
 
     const int lookback = 1;
 
-    double distacc = 0.0;
+    process_t distacc = 0.0;
 
     for (int i = count; i >= 0; i -= lookback) {
 
         bool resetThis = phaseReset;
-        
+
         if (bandlimited) {
             if (resetThis) {
                 if (i > bandlow && i < bandhigh) {
@@ -747,24 +824,24 @@ RubberBandStretcher::Impl::modifyChunk(size_t channel,
             }
         }
 
-        double p = cd.phase[i];
-        double perr = 0.0;
-        double outphase = p;
+        process_t p = cd.phase[i];
+        process_t perr = 0.0;
+        process_t outphase = p;
 
-        double mi = maxdist;
+        process_t mi = maxdist;
         if (i <= limit0) mi = 0.0;
         else if (i <= limit1) mi = 1.0;
         else if (i <= limit2) mi = 3.0;
 
         if (!resetThis) {
 
-            double omega = (2 * M_PI * m_increment * i) / (sz * cd.oversample);
+            process_t omega = (2 * M_PI * m_increment * i) / (m_fftSize);
 
-            double pp = cd.prevPhase[i];
-            double ep = pp + omega;
+            process_t pp = cd.prevPhase[i];
+            process_t ep = pp + omega;
             perr = princarg(p - ep);
 
-            double instability = fabs(perr - cd.prevError[i]);
+            process_t instability = fabs(perr - cd.prevError[i]);
             bool direction = (perr > cd.prevError[i]);
 
             bool inherit = false;
@@ -780,10 +857,10 @@ RubberBandStretcher::Impl::modifyChunk(size_t channel,
                 }
             }
 
-            double advance = outputIncrement * ((omega + perr) / m_increment);
+            process_t advance = outputIncrement * ((omega + perr) / m_increment);
 
             if (inherit) {
-                double inherited =
+                process_t inherited =
                     cd.unwrappedPhase[i + lookback] - cd.prevPhase[i + lookback];
                 advance = ((advance * distance) +
                            (inherited * (maxdist - distance)))
@@ -829,18 +906,16 @@ RubberBandStretcher::Impl::formantShiftChunk(size_t channel)
 
     ChannelData &cd = *m_channelData[channel];
 
-    double *const R__ mag = cd.mag;
-    double *const R__ envelope = cd.envelope;
-    double *const R__ dblbuf = cd.dblbuf;
+    process_t *const R__ mag = cd.mag;
+    process_t *const R__ envelope = cd.envelope;
+    process_t *const R__ dblbuf = cd.dblbuf;
+
+    const int sz = m_fftSize;
+    const int hs = sz / 2;
+    const process_t factor = 1.0 / sz;
 
-    const int sz = m_windowSize;
-    const int hs = m_windowSize/2;
-    const double factor = 1.0 / sz;
-    
     cd.fft->inverseCepstral(mag, dblbuf);
 
-    v_scale(dblbuf, factor, sz);
-
     const int cutoff = m_sampleRate / 700;
 
 //    cerr <<"cutoff = "<< cutoff << ", m_sampleRate/cutoff = " << m_sampleRate/cutoff << endl;
@@ -852,7 +927,10 @@ RubberBandStretcher::Impl::formantShiftChunk(size_t channel)
         dblbuf[i] = 0.0;
     }
 
-    cd.fft->forward(dblbuf, envelope, 0);
+    v_scale(dblbuf, factor, cutoff);
+
+    process_t *spare = (process_t *)alloca((hs + 1) * sizeof(process_t));
+    cd.fft->forward(dblbuf, envelope, spare);
 
     v_exp(envelope, hs + 1);
     v_divide(mag, envelope, hs + 1);
@@ -861,7 +939,7 @@ RubberBandStretcher::Impl::formantShiftChunk(size_t channel)
         // scaling up, we want a new envelope that is lower by the pitch factor
         for (int target = 0; target <= hs; ++target) {
             int source = lrint(target * m_pitchScale);
-            if (source > int(m_windowSize)) {
+            if (source > hs) {
                 envelope[target] = 0.0;
             } else {
                 envelope[target] = envelope[source];
@@ -882,7 +960,8 @@ RubberBandStretcher::Impl::formantShiftChunk(size_t channel)
 }
 
 void
-RubberBandStretcher::Impl::synthesiseChunk(size_t channel)
+RubberBandStretcher::Impl::synthesiseChunk(size_t channel,
+                                           size_t shiftIncrement)
 {
     Profiler profiler("RubberBandStretcher::Impl::synthesiseChunk");
 
@@ -894,52 +973,62 @@ RubberBandStretcher::Impl::synthesiseChunk(size_t channel)
 
     ChannelData &cd = *m_channelData[channel];
 
-    double *const R__ dblbuf = cd.dblbuf;
+    process_t *const R__ dblbuf = cd.dblbuf;
     float *const R__ fltbuf = cd.fltbuf;
     float *const R__ accumulator = cd.accumulator;
     float *const R__ windowAccumulator = cd.windowAccumulator;
     
-    int sz = m_windowSize;
-    int hs = m_windowSize/2;
-    int i;
+    const int fsz = m_fftSize;
+    const int hs = fsz / 2;
 
+    const int wsz = m_sWindowSize;
 
     if (!cd.unchanged) {
 
+        // Our FFTs produced unscaled results. Scale before inverse
+        // transform rather than after, to avoid overflow if using a
+        // fixed-point FFT.
+        float factor = 1.f / fsz;
+        v_scale(cd.mag, factor, hs + 1);
+
         cd.fft->inversePolar(cd.mag, cd.phase, cd.dblbuf);
 
-        if (cd.oversample > 1) {
-
-            int bufsiz = sz * cd.oversample;
-            int hbs = hs * cd.oversample;
-            int offset = (bufsiz - sz) / 2;
-
-            for (i = 0; i < hbs; ++i) {
-                double tmp = dblbuf[i];
-                dblbuf[i] = dblbuf[i + hbs];
-                dblbuf[i + hbs] = tmp;
-            }
-            for (i = 0; i < sz; ++i) {
-                fltbuf[i] = float(dblbuf[i + offset]);
-            }
-        } else {
+        if (wsz == fsz) {
             v_convert(fltbuf, dblbuf + hs, hs);
             v_convert(fltbuf + hs, dblbuf, hs);
+        } else {
+            v_zero(fltbuf, wsz);
+            int j = fsz - wsz/2;
+            while (j < 0) j += fsz;
+            for (int i = 0; i < wsz; ++i) {
+                fltbuf[i] += dblbuf[j];
+                if (++j == fsz) j = 0;
+            }
         }
-
-        // our ffts produced unscaled results
-        float factor = 1.f / float(sz * cd.oversample);
-        v_scale(fltbuf, factor, sz);
     }
 
-    m_window->cut(fltbuf);
+    if (wsz > fsz) {
+        int p = shiftIncrement * 2;
+        if (cd.interpolatorScale != p) {
+            SincWindow<float>::write(cd.interpolator, wsz, p);
+            cd.interpolatorScale = p;
+        }
+        v_multiply(fltbuf, cd.interpolator, wsz);
+    }
 
-    v_add(accumulator, fltbuf, sz);
+    m_swindow->cut(fltbuf);
+    v_add(accumulator, fltbuf, wsz);
+    cd.accumulatorFill = wsz;
 
-    cd.accumulatorFill = m_windowSize;
-
-    float fixed = m_window->getArea() * 1.5f;
-    m_window->add(windowAccumulator, fixed);
+    if (wsz > fsz) {
+        // reuse fltbuf to calculate interpolating window shape for
+        // window accumulator
+        v_copy(fltbuf, cd.interpolator, wsz);
+        m_swindow->cut(fltbuf);
+        v_add(windowAccumulator, fltbuf, wsz);
+    } else {
+        m_swindow->add(windowAccumulator, m_awindow->getArea() * 1.5f);
+    }
 }
 
 void
@@ -952,11 +1041,9 @@ RubberBandStretcher::Impl::writeChunk(size_t channel, size_t shiftIncrement, boo
     float *const R__ accumulator = cd.accumulator;
     float *const R__ windowAccumulator = cd.windowAccumulator;
 
-    const int sz = m_windowSize;
+    const int sz = m_sWindowSize;
     const int si = shiftIncrement;
 
-    int i;
-    
     if (m_debugLevel > 2) {
         cerr << "writeChunk(" << channel << ", " << shiftIncrement << ", " << last << ")" << endl;
     }
@@ -988,6 +1075,13 @@ RubberBandStretcher::Impl::writeChunk(size_t channel, size_t shiftIncrement, boo
             cd.setResampleBufSize(reqSize);
         }
 
+#ifndef NO_THREADING
+#if defined HAVE_IPP && !defined USE_SPEEX
+        if (m_threaded) {
+            m_resamplerMutex.lock();
+        }
+#endif
+#endif
 
         size_t outframes = cd.resampler->resample(&cd.accumulator,
                                                   &cd.resamplebuf,
@@ -995,6 +1089,13 @@ RubberBandStretcher::Impl::writeChunk(size_t channel, size_t shiftIncrement, boo
                                                   1.0 / m_pitchScale,
                                                   last);
 
+#ifndef NO_THREADING
+#if defined HAVE_IPP && !defined USE_SPEEX
+        if (m_threaded) {
+            m_resamplerMutex.unlock();
+        }
+#endif
+#endif
 
         writeOutput(*cd.outbuf, cd.resamplebuf,
                     outframes, cd.outCount, theoreticalOut);
@@ -1035,7 +1136,7 @@ RubberBandStretcher::Impl::writeOutput(RingBuffer<float> &to, float *from, size_
 
     size_t startSkip = 0;
     if (!m_realtime) {
-        startSkip = lrintf((m_windowSize/2) / m_pitchScale);
+        startSkip = lrintf((m_sWindowSize/2) / m_pitchScale);
     }
 
     if (outCount > startSkip) {
@@ -1102,19 +1203,25 @@ RubberBandStretcher::Impl::available() const
 {
     Profiler profiler("RubberBandStretcher::Impl::available");
 
+#ifndef NO_THREADING
     if (m_threaded) {
         MutexLocker locker(&m_threadSetMutex);
         if (m_channelData.empty()) return 0;
     } else {
         if (m_channelData.empty()) return 0;
     }
+#endif
 
+#ifndef NO_THREADING
     if (!m_threaded) {
+#endif
         for (size_t c = 0; c < m_channels; ++c) {
             if (m_channelData[c]->inputSize >= 0) {
 //                cerr << "available: m_done true" << endl;
                 if (m_channelData[c]->inbuf->getReadSpace() > 0) {
-//                    cerr << "calling processChunks(" << c << ") from available" << endl;
+                    if (m_debugLevel > 1) {
+                        cerr << "calling processChunks(" << c << ") from available" << endl;
+                    }
                     //!!! do we ever actually do this? if so, this method should not be const
                     // ^^^ yes, we do sometimes -- e.g. when fed a very short file
                     bool any = false, last = false;
@@ -1122,7 +1229,9 @@ RubberBandStretcher::Impl::available() const
                 }
             }
         }
+#ifndef NO_THREADING
     }
+#endif
 
     size_t min = 0;
     bool consumed = true;
@@ -1165,6 +1274,17 @@ RubberBandStretcher::Impl::retrieve(float *const *output, size_t samples) const
         }
     }
 
+    if ((m_options & OptionChannelsTogether) && (m_channels >= 2)) {
+        for (size_t i = 0; i < got; ++i) {
+            float mid = output[0][i];
+            float side = output[1][i];
+            float left = mid + side;
+            float right = mid - side;
+            output[0][i] = left;
+            output[1][i] = right;
+        }
+    }            
+
     return got;
 }
 
